// ParticleEngine.cpp: implementation of the ParticleEngine class.
//
//////////////////////////////////////////////////////////////////////

#include "FullParticleEngine.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FullParticleEngine::FullParticleEngine()
{
	physique = new ParticlePhysique();
	generator = new LoutreExplosion();
	particle = NULL;
	texture = NULL;
	particleNumber = 0;
	finish = false;
}

FullParticleEngine::~FullParticleEngine()
{
	delete physique;
	delete generator;
	if (particle != NULL)
	{
		delete[] particle;
	}
}

FullParticleEngine::ParticlePhysique::ParticlePhysique()
{
	physique.setCoord(0,-0.89,0); // la physique de base est celle de la terre
}


FullParticleEngine::ParticleGenerator::ParticleGenerator()
{
	maxGeneration = 10;
	generation = 0;
}


bool FullParticleEngine::Particle::isAlive()
{
	return (life <= maxLife);
}

void FullParticleEngine::frame(float elapsedTime)
{
	elapsedTime *= 0.001;
	finish = true;
	physique->frame(elapsedTime);	// on change la physique des particules
	if (generator->previousTime >= generator->generationDelay)
	{
		generator->generation = 0;	// on rinitialise le generateur
	}
	for (int i = 0; i < particleNumber; i++)
	{
		particle[i].frame(elapsedTime);
		if (particle[i].isAlive())
		{
			finish = false;
		}
	}
	generator->previousTime += elapsedTime;
}

void FullParticleEngine::Particle::frame(float elapsedTime)
{
	if (isAlive())
	{
		// puis on update la dure de vie de la particule
		life+= elapsedTime;
		// on ajoute le vecteur physique a la direction
		direction += physique->physique*elapsedTime;
		// on normalise la direction
		direction.normalisation();
		// on met a jour la position de la particule
		pos += direction*elapsedTime*speed;
		// et on met a jour l'alpha de la particule
		couleur[0] = beginCouleur[0] + ((life/maxLife))*(endCouleur[0] - beginCouleur[0]);
		couleur[1] = beginCouleur[1] + ((life/maxLife))*(endCouleur[1] - beginCouleur[1]);
		couleur[2] = beginCouleur[2] + ((life/maxLife))*(endCouleur[2] - beginCouleur[2]);
		couleur[3] = beginCouleur[3] + ((life/maxLife))*(endCouleur[3] - beginCouleur[3]);
	}
	else
	{
		// la particule est morte, on essaye de la regenerer (si le generateur a depass
		// le nombre max de particules genere, il ne la regenerera pas a ce tour mais au 
		// tour suivant.)
		
		//generator->generate(this);// on ne veut pas regenerer la particule
	}

}

void FullParticleEngine::setGenerator(FullParticleEngine::ParticleGenerator *gen)
{
	delete generator;
	generator = gen;
	for (int i = 0; i < particleNumber; i++)
	{
		particle[i].generator = generator;
	}

}

void FullParticleEngine::setPhysique(FullParticleEngine::ParticlePhysique *phy)
{
	delete physique;
	physique = phy;
	for (int i = 0; i < particleNumber; i++)
	{
		particle[i].physique = physique;
	}
}

void FullParticleEngine::createParticles(int number)
{
	if (particle == NULL)
	{
		delete[] particle;
	}
	particleNumber = number;
	particle = new Particle[particleNumber];
	generator->generation = 0;
	for (int i = 0; i < particleNumber; i++)
	{
		particle[i].generator = generator;
		generator->generate(&particle[i]);
		particle[i].physique = physique;
	}
}

void FullParticleEngine::setParticles(FullParticleEngine::Particle *part, int number)
{
	if (particle == NULL)
	{
		delete[] particle;
	}
	particleNumber = number;
	particle = part;
	for (int i = 0; i < particleNumber; i++)
	{
		particle[i].generator = generator;
		generator->generate(&particle[i]);
		particle[i].physique = physique;
	}
}

void FullParticleEngine::draw()
{
	bool blend;
	glBindTexture(GL_TEXTURE_2D, texture->getId());
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
	blend = glIsEnabled(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glEnable(GL_BLEND);
	glDepthMask(0);
	
	float matrice[16];
	float X[3];
	float Y[3];
	glGetFloatv(GL_MODELVIEW_MATRIX,matrice);
	X[0] = matrice[0];
	X[1] = matrice[4];
	X[2] = matrice[8];
	Y[0] = matrice[1];
	Y[1] = matrice[5];
	Y[2] = matrice[9];

	glBegin(GL_QUADS);
	for (int i = 0; i < particleNumber; i++)
	{
		if (particle[i].isAlive())
		{
			particle[i].draw(X,Y);
		}
	}
	glEnd();
	glPopMatrix();
	if (!blend)
	{
		glDisable(GL_BLEND);
	}
	glDepthMask(1);
}

void FullParticleEngine::Particle::draw(float *X, float *Y)
{
	float size2 = size /2;
	float vertice[4][3];
	vertice[0][0]=pos.getX()+(-X[0]+Y[0])*size2;
	vertice[0][1]=pos.getY()+(-X[1]+Y[1])*size2;
	vertice[0][2]=pos.getZ()+(-X[2]+Y[2])*size2;

	vertice[1][0]=pos.getX()+(-X[0]-Y[0])*size2;
	vertice[1][1]=pos.getY()+(-X[1]-Y[1])*size2;
	vertice[1][2]=pos.getZ()+(-X[2]-Y[2])*size2;

	vertice[2][0]=pos.getX()+(+X[0]+Y[0])*size2;
	vertice[2][1]=pos.getY()+(+X[1]+Y[1])*size2;
	vertice[2][2]=pos.getZ()+(+X[2]+Y[2])*size2;

	vertice[3][0]=pos.getX()+(+X[0]-Y[0])*size2;
	vertice[3][1]=pos.getY()+(+X[1]-Y[1])*size2;
	vertice[3][2]=pos.getZ()+(+X[2]-Y[2])*size2;

	//glColor4f(couleur.getRed(),couleur.getGreen(),couleur.getBlue(),couleur.getAlpha());
	glColor4fv(couleur);
	glTexCoord2f(0,0);glVertex3fv(vertice[1]); 
	glTexCoord2f(1,0);glVertex3fv(vertice[3]);
	glTexCoord2f(1,1);glVertex3fv(vertice[2]); 
	glTexCoord2f(0,1);glVertex3fv(vertice[0]); 
}



void LoutreExplosion::generate(FullParticleEngine::Particle *particle)
{
	if (generation < maxGeneration/4)
	{
		particle->speed = fabs(sin((float)rand()))*100;
	}
	else
	{
		particle->speed = fabs(sin((float)rand()))*300;
	}
	
	particle->pos = this->pos;
	particle->direction.setCoord(cos((float)rand()),cos((float)rand()),cos((float)rand()));
	particle->direction.normalisation();
	particle->direction.setOneCoord(1,fabs(particle->direction.getY()));
	if (generation == 1)
	{
		particle->direction.setCoord(particle->direction.getX()/2,particle->direction.getY()*10,particle->direction.getZ()/2);
		particle->direction.normalisation();
		particle->speed = 600;
	}
	particle->beginCouleur[0] = 0.0f;
	particle->beginCouleur[1] = 1.0f;
	particle->beginCouleur[2] = 0.0f;
	particle->beginCouleur[3] = 1.0f;
	particle->endCouleur[0] = 0.0f;
	particle->endCouleur[1] = 1.0f;
	particle->endCouleur[2] = 0.0f;
	particle->endCouleur[3] = 0.0f;

	particle->maxLife = 3;
	particle->life = 0;
	particle->size = 20;
	previousTime = 0;
	generation ++;


}

LoutreExplosion::LoutreExplosion()
{
	pos.setCoord(10,10,0);
	generation = 0;
	maxGeneration = 100;
	generationDelay = 0;
	previousTime = 0;

}

LoutreExplosion::~LoutreExplosion()
{

}

void FireRay::generate(FullParticleEngine::Particle *particle)
{
	particle->pos = pos + ((impact - pos)*((float)generation/(float)maxGeneration));
	particle->direction = (impact - pos);
	particle->direction.normalisation();
	particle->direction *= 5;
	particle->speed = 100;
	if (red)
	{
		particle->beginCouleur[0] = 1.0f;
		particle->beginCouleur[1] = 0.0f;
		particle->beginCouleur[2] = 0.0f;
		particle->beginCouleur[3] = 1.0f;
		particle->endCouleur[0] = 1.0f;
		particle->endCouleur[1] = 0.0f;
		particle->endCouleur[2] = 0.0f;
		particle->endCouleur[3] = 0.0f;
	}
	else
	{
		particle->beginCouleur[0] = 0.0f;
		particle->beginCouleur[1] = 1.0f;
		particle->beginCouleur[2] = 0.0f;
		particle->beginCouleur[3] = 1.0f;
		particle->endCouleur[0] = 0.0f;
		particle->endCouleur[1] = 1.0f;
		particle->endCouleur[2] = 0.0f;
		particle->endCouleur[3] = 0.0f;
	}
	particle->maxLife = 0.5;
	particle->life = 0;
	particle->size = 10;
	previousTime = 0;
	generation ++;
}

FireRay::FireRay(RULES_Vector pos, RULES_Vector max, bool r)
{
	this->pos = pos;
	this->impact = max;
	generation = 0;
	maxGeneration = 100;
	generationDelay = 0;
	previousTime = 0;
	red = r;
}

FireRay::~FireRay()
{
}

void BloodEngine::generate(FullParticleEngine::Particle *particle)
{
	particle->pos = this->pos;
	particle->direction= direction*1;
	RULES_Vector random;
	random.setCoord(cos((float)rand())/8.0f,cos((float)rand())/8.0f,cos((float)rand())/8.0f);
	particle->direction += random;
	particle->direction.normalisation();
	particle->beginCouleur[0] = 0.5f;
	particle->beginCouleur[1] = 0.0f;
	particle->beginCouleur[2] = 0.0f;
	particle->beginCouleur[3] = 1.0f;
	particle->endCouleur[0] = 1.0f;
	particle->endCouleur[1] = 0.0f;
	particle->endCouleur[2] = 0.0f;
	particle->endCouleur[3] = 0.0f;

	particle->speed = fabs(sin((float)rand()))*400;
	particle->maxLife = 2;
	particle->life = 0;
	particle->size = 20;
	previousTime = 0;
	generation ++;

}

BloodEngine::BloodEngine(RULES_Vector dir)
{
	generation = 0;
	maxGeneration = 100;
	generationDelay = 0;
	previousTime = 0;
	direction = dir;
}

BloodEngine::~BloodEngine()
{
}

void LoutreFlash::generate(FullParticleEngine::Particle *particle)
{
	particle->direction.setCoord(0,1,0);
	RULES_Vector random;
	random.setCoord(cos((float)rand())*3,0,cos((float)rand())*3);
	particle->pos = this->pos;
	particle->pos += random;
	particle->beginCouleur[0] = 0.1f;
	particle->beginCouleur[1] = 0.1f;
	particle->beginCouleur[2] = 1.0f;
	particle->beginCouleur[3] = 1.0f;
	particle->endCouleur[0] = 0.0f;
	particle->endCouleur[1] = 0.0f;
	particle->endCouleur[2] = 1.0f;
	particle->endCouleur[3] = 0.0f;

	particle->speed = 300 * ((float)generation/(float)maxGeneration) + 200;
	particle->maxLife = 0.5 + ((float)generation/(float)maxGeneration)*3.5;
	particle->life = 0;
	particle->size = 30;
	previousTime = 0;
	generation ++;

}

LoutreFlash::LoutreFlash()
{
	generation = 0;
	maxGeneration = 40;
	generationDelay = 0;
	previousTime = 0;
}

LoutreFlash::~LoutreFlash()
{
}

