// ParticleEngine.h: interface for the ParticleEngine class.
//
//////////////////////////////////////////////////////////////////////

#ifndef FULLPARTICLEENGINE_H
#define FULLPARTICLEENGINE_H

#ifdef WIN32
#include <windows.h>
#endif
#include <GL/gl.h>
#include "RULES_Vector.h"
#include "RULES_Texture.h"
/**
 * la classe de moteur de particule
 *
 * Cette classe contient 3 classes internes :
 * <BR>Particle : une particule de base avec une position,une direction et une physique
 * une vie, une vie max et un generateur.
 * a chaque frame, la particule s'auto update et si elle est morte, elle s'auto regerere
 * grace au particle gererator... ca permet au moteur de particule de juste avoir a verifier
 * les particule sans s'occuper des mechanismes de mouvements.
 * <BR><BR>ParticleGenerator : classe qui initialise les particules
 * <BR><BR>ParticlePhysique : classe qui gere le vecteur physique des particules.
 */
class FullParticleEngine  
{
public:
	/**
	 * methode appele pour afficher le moteur de particules
	 */ 
	void draw();

	/**
	 * methode appale pour updater le moteur de particules.
	 *
	 * toutes les particules du moteur vont se mettre a jour.
	 * @param elapsedTime: le temps ecoul depuis la frame precedente (temps exprim en secondes)
	 */
	void frame(float elapsedTime);
	class ParticleGenerator;
	class ParticlePhysique;
	
	/**
	 * la classe decrivant une particule d'un moteur.
	 *
	 * Une particule est une entit se deplacant dans l'espace, avec des proprit physique et une dure de vie limite.
	 * Quand une particule est morte, elle s'auto regenere en fonction du generator qui lui est donn.
	 */
	class Particle
	{
	public:
		/**
		 * methode utilis pour dessiner une particule en billboarding
		 * @param X: l'axe X du billboarding
		 * @param Y: l'axe Y du billboarding
		 */
		void draw(float *X, float *Y);

		/**
		 * methode qui met a jour la particule
		 * @param elapsedTime: le temps ecoul en secondes
		 */
		void frame(float elapsedTime);

		/**
		 * permet de savoir si la particule est encore en vis ou pas.
		 * @return bool: true si la particule est encore en vie, false sinon
		 */
		bool isAlive();

		/**
		 * la position de la particule dans l'espace
		 */
		RULES_Vector pos;		// la position de la particule

		/**
		 * la direction de la particule
		 */
		RULES_Vector direction;	// la direction de la particule

		/**
		 * le getionnaire de physique de la particule
		 */
		ParticlePhysique * physique;	// la physique de la particule

		/**
		 * la couleur de la particule a une frame donne
		 */
		float couleur[4];		// la couleur de la particule au temps t

		/**
		 * la couleur de debut de la particule
		 */
		float beginCouleur[4];	// la couleur de la particule au depart

		/**
		 * la couleur finale de la particule
		 */
		float endCouleur[4];	// la couleur de la particule a la fin

		/**
		 * l'age de la particule, en secondes
		 */
		float life;			// la vie de la particule

		/**
		 * l'age maximum de la particule (en secondes)
		 */
		float maxLife;		// la vie max de la particule

		/**
		 * la taille de la particule
		 */
		float size;			// la taille de la particule

		/**
		 * la vitesse de la particule
		 */
		float speed;		// la vitesse de la particule

		/**
		 * le generateur de la particule. Quand la particule est morte, elle passe par le generateur
		 * qui la regenere.
		 */
		ParticleGenerator * generator;	// le generateur de particule.

		/**
		 * constructeur par defaut
		 */
		Particle()
		{
			life = 0;
			maxLife = -1;
			size = 1;
			speed = 1;
			beginCouleur[0] = 1.0f;
			beginCouleur[1] = 1.0f;
			beginCouleur[2] = 1.0f;
			beginCouleur[3] = 1.0f;
			couleur[0] = 1.0f;
			couleur[1] = 1.0f;
			couleur[2] = 1.0f;
			couleur[3] = 1.0f;
			endCouleur[0] = 1.0f;
			endCouleur[1] = 1.0f;
			endCouleur[2] = 1.0f;
			endCouleur[3] = 0.0f;
		};
		/**
		 * destructeur par defaut
		 */
		virtual ~Particle(){};

	};

	/**
	 * Classe gerant la regeneration des particules.
	 *
	 * chaque moteur de particule a un particle generator qui regenere toutes les particules mortes.
	 * Le but de cette classe est de servire de classe de base  de nombreux type de generator qui implementent
	 * de multiple type de moteur de particules.
	 */
	class ParticleGenerator
	{
	public:
		/**
		 * constructeur par defaut.
		 */
		ParticleGenerator();

		/**
		 * destructeur par defaut
		 */
		virtual ~ParticleGenerator(){};

		/**
		 * methode virtuelle servant a regenerer les particules.
		 * @param particle: pointeur sur la particule qui doit etre regenerer
		 */
		virtual void generate(Particle * particle){};

		/**
		 * le nombre maximum de particules que doit regenerer le generator par lapse de temp (voir generationDelay)
		 */
		int maxGeneration;	// le nombre max de generation par frame

		/**
		 * le nombre de generation durant le lapse de temps courant
		 */
		int generation;		// le nombre de generation a la frame courante

		/**
		 * la position du generateur. cela permet par exemple de faire bouger le moteur de particules
		 * toute en faisant bouger le generateur differament...
		 */
		RULES_Vector pos;		// la position du generateur

		/**
		 * le delais durant lequel le generateur ne peut pas depasser le nombre maximum de generations.
		 *
		 * Ce delais sert par exemple a dire que le moteur de particule ne doit pas generer plus de 100 particule
		 * durant un delai de 1 secondes
		 */
		float generationDelay;

		/**
		 * le temps precedent.
		 */
		float previousTime;
	};

	/**
	 * la classe gerant la physique appliqu a une particule a chaque frame.
	 *
	 * Cette classe parmet de faire varier le comportement d'une particule en fonction du temps.
	 * Par exemple une particule lanc dans une direction peut etre devi par du vent et par sa masse.
	 */
	class ParticlePhysique
	{
	public:
		/**
		 * le constructeur par defaut
		 */
		ParticlePhysique();

		/**
		 * le destructeur par defaut
		 */
		virtual ~ParticlePhysique(){};

		/**
		 * le vecteur de physique
		 */
		RULES_Vector physique;

		/**
		 * la mathode permettant de mettre a jour le moteur de physique
		 */
		virtual void frame(float elapsedTime){};
	};

	///////////////////////////////////////////////////////////////////	
	
	/**
	 * le constructeur par defaut
	 */
	FullParticleEngine();

	/**
	 * le destructeur par defaut
	 */
	virtual ~FullParticleEngine();

	/**
	 * methode definissant le generator utilis par le moteur.
	 *
	 * toute les particules du moteur sont mise a jour au niveau du generator
	 * @param gen: le generator qui sera appliqu a toutes les particules
	 */
	void setGenerator(ParticleGenerator * gen);

	/**
	 * methode retournant le generator du moteur de particules
	 * @return ParticleGenerator: le particle generator du moteur
	 */
	ParticleGenerator * getGenerator(){return generator;};

	/**
	 * methode deffinissant la physique a applique au particules du moteur
	 * @param phy: le moteur de physique du moteur de particules
	 */
	void setPhysique(ParticlePhysique * phy);

	/**
	 * methode retournant le moteur de physique.
	 * @return ParticlePhysique: le moteur de physique du moteur de particule
	 */
	ParticlePhysique * getPhysique(){return physique;};

	/**
	 * methode crant uhn certain nombre de particules dans le moteur.
	 * @param number: le nombre de particules a cre dans le moteur.
	 */
	void createParticles(int number);

	/**
	 * methode permettant de definire les particules du moteur.
	 * 
	 * cette methode est a utilise si on veut cre ses propres particules (hritant de particules de bases) dans
	 * le moteur.
	 * @param part: les particules a mettre dans le moteur.<B>ATTENTION</B> Les particules sont possd par le moteur,
	 * c'est donc lui qui les dtruits, il n'en fait pas de copies...
	 * @param number: le nombre de particules a ajouter au moteur
	 */
	void setParticles(Particle * part, int number);

	/**
	 * methode definissant la texture a appliquer au particules.
	 * @param tex: la texture  appliquer
	 */
	void setTexture(RULES_Texture * tex){texture = tex;};

	/**
	 * methode permettant de savoir si l'ensemble des particules sont mortes
	 * @return bool: true si toutes les particules sont mortes, false sinon
	 */
	bool isFinish(){return finish;};
	
private:
	/**
	 * le tableau de particules
	 */
	Particle * particle;

	/**
	 * le generator de particules
	 */
	ParticleGenerator * generator;

	/**
	 * le moteur de physique des particules
	 */
	ParticlePhysique * physique;

	/**
	 * le nombre de particules
	 */
	int particleNumber;

	/**
	 * la texture des particules
	 */
	RULES_Texture * texture;

	/**
	 * permet de savoir si toutes les particules sont mortes
	 */
	bool finish;

};

/**
 * definition de la classe qui gere les explosions des loutres
 */
class LoutreExplosion : public FullParticleEngine::ParticleGenerator
{
public:
	/**
	 * methode regenerant une particules donne
	 * @param particle: la particule a regenerer
	 */
	void generate(FullParticleEngine::Particle *particle);
	/**
	 * constructeur par defaut
	 */
	LoutreExplosion();

	/**
	 * destructeur par defaut
	 */
	virtual ~LoutreExplosion();

};

/**
 * classe gerant les tirs de l'arme
 */
class FireRay : public FullParticleEngine::ParticleGenerator
{
private:
	/**
	 * le point d'impacte du tir
	 */
	RULES_Vector impact;
	/**
	 * permet de savoir si le tire est rouge ou vert
	 */
	bool red;
public:
	/**
	 * methode regenerant une particules donne
	 * @param particle: la particule a regenerer
	 */
	void generate(FullParticleEngine::Particle *particle);
	/**
	 * constructeur par defaut
	 */
	FireRay(RULES_Vector pos, RULES_Vector max, bool r);
	/**
	 * destructeur par defaut
	 */
	virtual ~FireRay();

};

/**
 * classe gerant les gicle de sang des loutres
 */
class BloodEngine : public FullParticleEngine::ParticleGenerator
{
private:
	/**
	 * la direction dans laquelle le sang gicle
	 */
	RULES_Vector direction;
public:
	/**
	 * methode regenerant une particules donne
	 * @param particle: la particule a regenerer
	 */
	void generate(FullParticleEngine::Particle *particle);
	/**
	 * constructeur par defaut
	 */
	BloodEngine(RULES_Vector dir);
	/**
	 * destructeur par defaut
	 */
	virtual ~BloodEngine();

};

/**
 * classe gerant l'impacte d'ne loutre sur l'immeuble
 */
class LoutreFlash : public FullParticleEngine::ParticleGenerator
{
public:
	/**
	 * methode regenerant une particules donne
	 * @param particle: la particule a regenerer
	 */
	void generate(FullParticleEngine::Particle *particle);
	/**
	 * constructeur par defaut
	 */
	LoutreFlash();
	/**
	 * destructeur par defaut
	 */
	virtual ~LoutreFlash();

};

#endif
