// LoutreManager.h: interface for the LoutreManager class.
//
//////////////////////////////////////////////////////////////////////

#ifndef LOUTREMANAGER_H
#define LOUTREMANAGER_H

#define UBER_LOUTRE_NUMBER 15

#include <vector>
#include "Loutre.h"
#include "Immeuble.h"
#include "Frustum.h"	// Added by ClassView
#include "MotionBlur.h"
#include "SoundManager.h"
#include "FullParticleEngine.h"
#include <list>

class LoutreManager  
{
private:
	/**
	 * variable permettant de savoir si une loutre est dans l'immeuble
	 */
	bool loutreOnImmeuble;

	/**
	 * le nombre de loutres a gerer
	 */
	int loutreNumber;

	/**
	 * le frustum de vue, utilis pour savoir quelles loutres sont visibles ou pas
	 */
	Frustum frustum;

	/**
	 * le rayon de creation des loutres
	 */
	float creationRadius;

	/**
	 * les loutres
	 */
	Loutre * loutres;
	/**
	 * l'immeuble. a la base seul un immeuble est prevu mais comme ca on pourra en rajouter...
	 */
	Immeuble * immeuble;

	/**
	 * le nombre de loutres genere depuis la derniere uberLoutre
	 */ 
	int generatedLoutre;

	/**
	 * la liste qui gere tout les moteurs de particules
	 */
	std::list<FullParticleEngine *>particleEngines;

	/**
	 * la texture de particule utilise pour les moteurs de particule de tire et d'explosion
	 */
	RULES_Texture *fireParticleTexture;

	/**
	 * la texture de particule utilise pour le moteur de particules de sang.
	 */
	RULES_Texture *bloodParticleTexture;

public:
	/**
	 * methode utilise pour connaitre le nombre de loutres
	 * @return int: le nombre de loutre du manager
	 */
	int getLoutreNumber();

	/**
	 * permet de remettre a 0 le temps lorsqu'on est sortie du jeu (dans le menu) et qu'on y revient. si on
	 * ne remet pas le temps a 0, le loutre manager considere que les loutres doivent effectuer un grand deplacement
	 * alors que le jeu etait en fait en pause...
	 */
	void resetTime();

	/**
	 * permet de savoir si une loutre est en collision avec l'immeuble
	 */
	bool collisionWithImmeuble();

	/**
	 * methode de debug utilis pour tuer toutes les loutres
	 */ 
	void killAll();

	/**
	 * definit l'immeuble du jeu
	 * @param immeuble: l'immeuble du jeu
	 */
	void setImmeuble(Immeuble * immeuble);

	/**
	 * definit les loutres du jeu
	 * <B>/!\ATTENTION/!\</B> les loutres passe en parametre sont possed par le loutre manager, c'est a dire qu'il les
	 * detruits dans son destructeur !!!
	 * @param loutre: pointeur sur un tableau de loutres
	 * @param number: le nombre de loutre presentes dans le tableau de loutre.
	 */
	void setLoutres(Loutre * loutre,int number);

	/**
	 * definit le rayon de creation des loutres.
	 * Les loutres seront cre apres ce rayon.
	 * @param radius: le rayon de creation des loutres
	 */
	void setCreationRadius(float radius);

	/**
	 * methode retournant l'immeuble
	 * @return Immeuble: l'immeuble du jeu
	 */
	Immeuble * getImmeuble();

	/**
	 * methode retournant les loutres du jeu
	 * @return Loutre: les loutres du jeu.
	 */
	Loutre * getLoutre(int index);

	/**
	 * methode permettant de gerer les loutres en fonction du niveau de difficult (fait bouger les loutres
	 * verifie leur visibilit, les collision avec l'immeuble ...)
	 * @param difficulty: le niveau de difficult du jeu
	 */
	void manageLoutres(int difficulty);

	/**
	 * constructeur par defaut
	 */
	LoutreManager();

	/**
	 * destructeur par defaut
	 */
	virtual ~LoutreManager();

	/**
	 * ajoute un moteur de particule d'explosion de loutre a la position donne
	 * @param pos la position du moteur de particules
	 */
	void addLoutreExplosionAtPos(RULES_Vector pos);

	/**
	 * ajoute un moteur de particule d'explosion de loutre a la position donne
	 * @param pos la position du moteur de particules
	 */
	void addLoutreFlashAtPos(RULES_Vector pos);

	/**
	 * ajoute un moteur de particules de tire allant de la position donn au point d'impact donn
	 * @param pos: la position du moteur de particules
	 * @param impact: le point d'impact
	 */
	void addFireRayAtPos(RULES_Vector pos,RULES_Vector impact, bool red);

	/**
	 * ajoute un moteur de particules de sang a la position donn avec le direction donn
	 * @param pos la position du moteur de particules
	 * @param direction: la direction dans laquelle le sang va gicler...
	 */
	void addBloodExplosionAtPos(RULES_Vector pos,RULES_Vector direction);

	/**
	 * gere l'ensemble des moteurs de particules.
	 * @param elapsedTime: le temps ecoule a la frame
	 */
	void manageParticles(float elapsedTime);

	/**
	 * dessine l'ensemble des moteur de particules du jeu.
	 */
	void drawParticles();

	/**
	 * supprime tout les moteurs de particules.
	 */
	void resetParticleEngine();
};

#endif 